const { declare } = require('@babel/helper-plugin-utils');

const defaultTargets = {};

function buildTargets({ additionalTargets }) {
  return { ...defaultTargets, ...additionalTargets };
}

module.exports = declare((api, options) => {
  // see docs about api at https://babeljs.io/docs/en/config-files#apicache
  api.assertVersion('^7.0.0');

  const { commonJS, modules, targets = buildTargets(options), react, removePropTypes } = options;

  if (typeof modules !== 'undefined' && typeof modules !== 'boolean' && modules !== 'auto') {
    throw new TypeError(
      '@nuofe/babel-preset-ndk only accepts `true`, `false`, or `"auto"` as the value of the "modules" option',
    );
  }

  const computedModulesOption = modules === false ? false : 'auto';

  const debug = typeof options.debug === 'boolean' ? options.debug : false;
  const development =
    typeof options.development === 'boolean'
      ? options.development
      : api.cache.using(() => process.env.NODE_ENV === 'development');

  return {
    presets: [
      [
        require('@babel/preset-env'),
        {
          debug,
          corejs: 3,
          exclude: ['transform-template-literals'],
          loose: true,
          modules: computedModulesOption,
          shippedProposals: true,
          targets,
          useBuiltIns: 'usage',
        },
      ],
      react && [require('@babel/preset-react'), { development, useBuiltIns: true }],
      [require('@babel/preset-typescript')],
    ].filter(Boolean),
    plugins: [
      [
        require('@babel/plugin-transform-runtime'),
        {
          absoluteRuntime: false,
          corejs: false,
          helpers: true,
          regenerator: true,
          useESModules: !computedModulesOption,
        },
      ],
      commonJS && [
        require('@babel/plugin-transform-modules-commonjs'),
        {
          loose: true,
        },
      ],
      removePropTypes && [
        require('babel-plugin-transform-react-remove-prop-types'),
        {
          mode: 'wrap',
          additionalLibraries: ['airbnb-prop-types'],
          ignoreFilenames: ['node_modules'],
          ...removePropTypes,
        },
      ],
      [require('@babel/plugin-transform-template-literals'), { spec: true }],
      [require('@babel/plugin-transform-property-mutators')],
      [require('@babel/plugin-proposal-decorators'), { legacy: true }],
    ].filter(Boolean),
  };
});
